@@ -78,6 +78,7 @@ struct _GVfsBackendSmb
   GMountSource *mount_source; /* Only used/set during mount */
   int mount_try;
   gboolean mount_try_again;
+  gboolean mount_cancelled;
 	
   gboolean password_in_keyring;
   GPasswordSave password_save;
@@ -94,6 +95,13 @@ static char *default_workgroup = NULL;
 
 G_DEFINE_TYPE (GVfsBackendSmb, g_vfs_backend_smb, G_VFS_TYPE_BACKEND)
 
+static void set_info_from_stat (GVfsBackendSmb *backend,
+				GFileInfo *info,
+				struct stat *statbuf,
+				const char *basename,
+				GFileAttributeMatcher *matcher);
+
+
 static void
 g_vfs_backend_smb_finalize (GObject *object)
 {
@@ -158,6 +166,14 @@ auth_callback (SMBCCTX *context,
   if (backend->user)
     strncpy (username_out, backend->user, unmaxlen);
 
+  if (backend->mount_cancelled)
+    {
+      /*  Don't prompt for credentials, let smbclient finish the mount loop  */
+      strncpy (username_out, "ABORT", unmaxlen);
+      strncpy (password_out, "", pwmaxlen);
+      return;
+    }
+
   if (backend->mount_source == NULL)
     {
       /* Not during mount, use last password */
@@ -231,6 +247,7 @@ auth_callback (SMBCCTX *context,
 	    {
 	      strncpy (username_out, "ABORT", unmaxlen);
 	      strncpy (password_out, "", pwmaxlen);
+	      backend->mount_cancelled = TRUE;
 	      goto out;
 	    }
 	}
@@ -552,11 +569,12 @@ do_mount (GVfsBackend *backend,
   do
     {
       op_backend->mount_try_again = FALSE;
+      op_backend->mount_cancelled = FALSE;
 
       smbc_stat = smbc_getFunctionStat (smb_context);
       res = smbc_stat (smb_context, uri, &st);
       
-      if (res == 0 ||
+      if (res == 0 || op_backend->mount_cancelled ||
 	  (errno != EACCES && errno != EPERM))
 	break;
 
@@ -581,10 +599,17 @@ do_mount (GVfsBackend *backend,
     {
       /* TODO: Error from errno? */
       op_backend->mount_source = NULL;
-      g_vfs_job_failed (G_VFS_JOB (job),
-			G_IO_ERROR, G_IO_ERROR_FAILED,
-			/* translators: We tried to mount a windows (samba) share, but failed */
-			_("Failed to mount Windows share"));
+      
+      if (op_backend->mount_cancelled) 
+        g_vfs_job_failed (G_VFS_JOB (job),
+			  G_IO_ERROR, G_IO_ERROR_PERMISSION_DENIED,
+			  _("Password dialog cancelled"));
+      else
+        g_vfs_job_failed (G_VFS_JOB (job),
+			  G_IO_ERROR, G_IO_ERROR_FAILED,
+			  /* translators: We tried to mount a windows (samba) share, but failed */
+			  _("Failed to mount Windows share"));
+
       return;
     }
 
@@ -750,6 +775,33 @@ do_seek_on_read (GVfsBackend *backend,
   return;
 }
 
+static void
+do_query_info_on_read (GVfsBackend *backend,
+		       GVfsJobQueryInfoRead *job,
+		       GVfsBackendHandle handle,
+		       GFileInfo *info,
+		       GFileAttributeMatcher *matcher)
+{
+  GVfsBackendSmb *op_backend = G_VFS_BACKEND_SMB (backend);
+  struct stat st = {0};
+  int res, saved_errno;
+  smbc_fstat_fn smbc_fstat;
+
+  smbc_fstat = smbc_getFunctionFstat (op_backend->smb_context);
+  res = smbc_fstat (op_backend->smb_context, (SMBCFILE *)handle, &st);
+  saved_errno = errno;
+
+  if (res == 0)
+    {
+      set_info_from_stat (op_backend, info, &st, NULL, matcher);
+      
+      g_vfs_job_succeeded (G_VFS_JOB (job));
+    }
+  else
+    g_vfs_job_failed_from_errno (G_VFS_JOB (job), saved_errno);
+
+}
+
 static void
 do_close_read (GVfsBackend *backend,
 	       GVfsJobCloseRead *job,
@@ -1206,6 +1258,34 @@ do_seek_on_write (GVfsBackend *backend,
   return;
 }
 
+static void
+do_query_info_on_write (GVfsBackend *backend,
+			GVfsJobQueryInfoWrite *job,
+			GVfsBackendHandle _handle,
+			GFileInfo *info,
+			GFileAttributeMatcher *matcher)
+{
+  GVfsBackendSmb *op_backend = G_VFS_BACKEND_SMB (backend);
+  struct stat st = {0};
+  SmbWriteHandle *handle = _handle;
+  int res, saved_errno;
+  smbc_fstat_fn smbc_fstat;
+
+  smbc_fstat = smbc_getFunctionFstat (op_backend->smb_context);
+  res = smbc_fstat (op_backend->smb_context, handle->file, &st);
+  saved_errno = errno;
+
+  if (res == 0)
+    {
+      set_info_from_stat (op_backend, info, &st, NULL, matcher);
+      
+      g_vfs_job_succeeded (G_VFS_JOB (job));
+    }
+  else
+    g_vfs_job_failed_from_errno (G_VFS_JOB (job), saved_errno);
+
+}
+
 static void
 do_close_write (GVfsBackend *backend,
 		GVfsJobCloseWrite *job,
@@ -1381,13 +1461,8 @@ set_info_from_stat (GVfsBackendSmb *backend,
       else
 	{
 	  content_type = g_content_type_guess (basename, NULL, 0, NULL);
-	  
 	  if (content_type)
-	    {
-              icon = g_content_type_get_icon (content_type);
-              if (G_IS_THEMED_ICON (icon))
-                g_themed_icon_append_name (G_THEMED_ICON (icon), "text-x-generic");
-	    }
+	    icon = g_content_type_get_icon (content_type);
 	}
       
       if (content_type)
@@ -1504,12 +1579,25 @@ try_query_settable_attributes (GVfsBackend *backend,
 
   list = g_file_attribute_info_list_new ();
 
-  /* TODO: Add all settable attributes here */
-  /*
+  /* TODO: Add all settable attributes here -- bug #559586 */
+  /* TODO: xattrs support? */
+
   g_file_attribute_info_list_add (list,
-				  "smb:test",
-				  G_FILE_ATTRIBUTE_TYPE_UINT32);
-  */
+                                  G_FILE_ATTRIBUTE_TIME_MODIFIED,
+                                  G_FILE_ATTRIBUTE_TYPE_UINT64,
+                                  G_FILE_ATTRIBUTE_INFO_COPY_WITH_FILE |
+                                  G_FILE_ATTRIBUTE_INFO_COPY_WHEN_MOVED);
+
+#if 0
+/* FIXME: disabled; despite chmod is supported, it makes no sense on samba shares and
+          libsmbclient lacks proper API to read unix file modes.
+          The struct stat->st_mode member is used for special Windows attributes. */
+  g_file_attribute_info_list_add (list,
+                                  G_FILE_ATTRIBUTE_UNIX_MODE,
+                                  G_FILE_ATTRIBUTE_TYPE_UINT32,
+                                  G_FILE_ATTRIBUTE_INFO_COPY_WITH_FILE |
+                                  G_FILE_ATTRIBUTE_INFO_COPY_WHEN_MOVED);
+#endif
 
   g_vfs_job_query_attributes_set_list (job, list);
   g_vfs_job_succeeded (G_VFS_JOB (job));
@@ -1518,6 +1606,70 @@ try_query_settable_attributes (GVfsBackend *backend,
   return TRUE;
 }
 
+static void
+do_set_attribute (GVfsBackend *backend,
+                  GVfsJobSetAttribute *job,
+                  const char *filename,
+                  const char *attribute,
+                  GFileAttributeType type,
+                  gpointer value_p,
+                  GFileQueryInfoFlags flags)
+{
+  GVfsBackendSmb *op_backend;
+  char *uri;
+  int res, errsv;
+  struct timeval tbuf[2];
+  smbc_utimes_fn smbc_utimes;
+#if 0
+  smbc_chmod_fn smbc_chmod;
+#endif
+
+
+  op_backend = G_VFS_BACKEND_SMB (backend);
+
+  if (strcmp (attribute, G_FILE_ATTRIBUTE_TIME_MODIFIED) != 0
+#if 0
+      && strcmp (attribute, G_FILE_ATTRIBUTE_UNIX_MODE) != 0
+#endif
+      )
+    {
+      g_vfs_job_failed (G_VFS_JOB (job),
+                        G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
+                        _("Operation unsupported"));
+      return;
+    }
+
+  uri = create_smb_uri (op_backend->server, op_backend->share, filename);
+  res = -1;
+
+  if (strcmp (attribute, G_FILE_ATTRIBUTE_TIME_MODIFIED) == 0)
+    {
+      smbc_utimes = smbc_getFunctionUtimes (op_backend->smb_context);
+      tbuf[1].tv_sec = (*(guint64 *)value_p);  /* mtime */
+      tbuf[1].tv_usec = 0;
+      /* atime = mtime (atimes are usually disabled on desktop systems) */
+      tbuf[0].tv_sec = tbuf[1].tv_sec;  
+      tbuf[0].tv_usec = 0;
+      res = smbc_utimes (op_backend->smb_context, uri, &tbuf[0]);
+    }
+#if 0
+  else
+  if (strcmp (attribute, G_FILE_ATTRIBUTE_UNIX_MODE) == 0)
+    {
+      smbc_chmod = smbc_getFunctionChmod (op_backend->smb_context);
+      res = smbc_chmod (op_backend->smb_context, uri, (*(guint32 *)value_p) & 0777);
+    }
+#endif    
+
+  errsv = errno;
+  g_free (uri);
+
+  if (res != 0)
+    g_vfs_job_failed_from_errno (G_VFS_JOB (job), errsv);
+  else
+    g_vfs_job_succeeded (G_VFS_JOB (job));
+}
+
 static void
 do_enumerate (GVfsBackend *backend,
 	      GVfsJobEnumerate *job,
@@ -1904,12 +2056,14 @@ g_vfs_backend_smb_class_init (GVfsBackendSmbClass *klass)
   backend_class->open_for_read = do_open_for_read;
   backend_class->read = do_read;
   backend_class->seek_on_read = do_seek_on_read;
+  backend_class->query_info_on_read = do_query_info_on_read;
   backend_class->close_read = do_close_read;
   backend_class->create = do_create;
   backend_class->append_to = do_append_to;
   backend_class->replace = do_replace;
   backend_class->write = do_write;
   backend_class->seek_on_write = do_seek_on_write;
+  backend_class->query_info_on_write = do_query_info_on_write;
   backend_class->close_write = do_close_write;
   backend_class->query_info = do_query_info;
   backend_class->query_fs_info = do_query_fs_info;
@@ -1919,6 +2073,7 @@ g_vfs_backend_smb_class_init (GVfsBackendSmbClass *klass)
   backend_class->make_directory = do_make_directory;
   backend_class->move = do_move;
   backend_class->try_query_settable_attributes = try_query_settable_attributes;
+  backend_class->set_attribute = do_set_attribute;
 
 #ifdef HAVE_GCONF
   gclient = gconf_client_get_default ();
